-------------------------
-- Stagger, by Siweia
-------------------------
local _, ns = ...
local cfg = ns.cfg
if cfg.MyClass ~= "MONK" then return end
local cr, cg, cb = cfg.cc.r, cfg.cc.g, cfg.cc.b

-- APIs
local CreateBD = function(f, a, s)
	f:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeFile = cfg.glowTex, edgeSize = s or 3,
		insets = {left = s or 3, right = s or 3, top = s or 3, bottom = s or 3},
	})
	f:SetBackdropColor(0, 0, 0, a or .5)
	f:SetBackdropBorderColor(0, 0, 0)
end

local CreateSD = function(f, m, s, n)
	if f.Shadow then return end
	f.Shadow = CreateFrame("Frame", nil, f, "BackdropTemplate")
	f.Shadow:SetPoint("TOPLEFT", f, -m, m)
	f.Shadow:SetPoint("BOTTOMRIGHT", f, m, -m)
	f.Shadow:SetBackdrop({
		edgeFile = cfg.glowTex, edgeSize = s })
	f.Shadow:SetBackdropBorderColor(0, 0, 0, 1)
	f.Shadow:SetFrameLevel(n or f:GetFrameLevel())
	return f.Shadow
end

local CreateIF = function(f)
	CreateSD(f, 3, 3)
	f.Icon = f:CreateTexture(nil, "ARTWORK")
	f.Icon:SetAllPoints()
	f.Icon:SetTexCoord(unpack(cfg.TexCoord))
	f.CD = CreateFrame("Cooldown", nil, f, "CooldownFrameTemplate")
	f.CD:SetAllPoints()
	f.CD:SetReverse(true)
end

local CreateSB = function(f, spark)
	f:SetStatusBarTexture(cfg.normTex)
	f:SetStatusBarColor(cr, cg, cb)
	CreateSD(f, 3, 3)
	f.BG = f:CreateTexture(nil, "BACKGROUND")
	f.BG:SetAllPoints()
	f.BG:SetTexture(cfg.normTex)
	f.BG:SetVertexColor(cr, cg, cb, .2)
	if spark then
		f.Spark = f:CreateTexture(nil, "OVERLAY")
		f.Spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		f.Spark:SetBlendMode("ADD")
		f.Spark:SetAlpha(.8)
		f.Spark:SetPoint("TOPLEFT", f:GetStatusBarTexture(), "TOPRIGHT", -10, 10)
		f.Spark:SetPoint("BOTTOMRIGHT", f:GetStatusBarTexture(), "BOTTOMRIGHT", 10, -10)
	end
end

local SetMover = function(Frame, Text, key, Pos, w, h)
	if not StgDB[key] then StgDB[key] = {} end
	local Mover = CreateFrame("Frame", nil, UIParent, "BackdropTemplate")
	Mover:SetWidth(w or Frame:GetWidth())
	Mover:SetHeight(h or Frame:GetHeight())
	CreateBD(Mover)
	Mover.Text = Mover:CreateFontString(nil, "OVERLAY")
	Mover.Text:SetFont(unpack(cfg.Font))
	Mover.Text:SetPoint("CENTER")
	Mover.Text:SetText(Text)
	if not StgDB[key]["Mover"] then 
		Mover:SetPoint(unpack(Pos))
	else
		Mover:SetPoint(unpack(StgDB[key]["Mover"]))
	end
	Mover:EnableMouse(true)
	Mover:SetMovable(true)
	Mover:SetClampedToScreen(true)
	Mover:SetFrameStrata("HIGH")
	Mover:RegisterForDrag("LeftButton")
	Mover:SetScript("OnDragStart", function(self) self:StartMoving() end)
	Mover:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		local orig, _, tar, x, y = self:GetPoint()
		StgDB[key]["Mover"] = {orig, "UIParent", tar, x, y}
	end)
	Mover:Hide()
	Frame:ClearAllPoints()
	Frame:SetPoint("TOPLEFT", Mover)
	return Mover
end

local Numb = function(n)
	if (n >= 1e6) then
		return ("%.1fm"):format(n / 1e6)
	elseif (n >= 1e3) then
		return ("%.1fk"):format(n / 1e3)
	else
		return ("%.0f"):format(n)
	end
end

local CreateFS = function(f, size, text, classcolor, anchor, x, y)
	local fs = f:CreateFontString(nil, "OVERLAY")
	fs:SetFont(cfg.Font[1], size, cfg.Font[3])
	fs:SetText(text)
	fs:SetWordWrap(false)
	if classcolor then
		fs:SetTextColor(cr, cg, cb)
	end
	if (anchor and x and y) then
		fs:SetPoint(anchor, x, y)
	else
		fs:SetPoint("CENTER", 1, 0)
	end
	return fs
end

-- Style
local IconSize = cfg.IconSize
local bu, bar = {}
local function StaggerGo()
	if bar then bar:Show() return end

	bar = CreateFrame("StatusBar", "NDui_Stagger", UIParent)
	bar:SetSize(IconSize*5 + 20, cfg.BarHeight)
	bar:SetPoint("CENTER", 0, -200)
	bar:SetFrameStrata("HIGH")
	CreateSB(bar, true)
	bar:SetMinMaxValues(0, 100)
	bar:SetValue(0)
	bar.Count = CreateFS(bar, 16, "", false, "TOPRIGHT", 0, -7)

	local spells = {121253, 322101, 215479, 325092, 322507}
	for i = 1, 5 do
		bu[i] = CreateFrame("Frame", nil, bar)
		bu[i]:SetSize(IconSize, IconSize)
		bu[i]:SetFrameStrata("HIGH")
		CreateIF(bu[i])
		bu[i].Icon:SetTexture(GetSpellTexture(spells[i]))
		bu[i].Count = CreateFS(bu[i], 16, "")
		bu[i].Count:SetPoint("BOTTOMRIGHT", 4, -2)
		if i == 1 then
			bu[i]:SetPoint("BOTTOMLEFT", bar, "TOPLEFT", 0, 5)
		else
			bu[i]:SetPoint("LEFT", bu[i-1], "RIGHT", 5, 0)
		end
	end
	bu[1].Count:SetAllPoints()

	local Mover = SetMover(bar, NPE_MOVE, "Stagger", cfg.StaggerPos, bar:GetWidth(), 20)
	SlashCmdList["STAGGER"] = function(msg)
		if InCombatLockdown() then return end
		if msg:lower() == "reset" then
			wipe(StgDB["Stagger"])
			ReloadUI()
		else
			if Mover:IsVisible() then
				Mover:Hide()
			else
				Mover:Show()
			end
		end
	end
	SLASH_STAGGER1 = "/stg"
end

local function UpdateCooldown(button, spellID, texture)
	local charges, maxCharges, chargeStart, chargeDuration = GetSpellCharges(spellID)
	local start, duration = GetSpellCooldown(spellID)
	if charges and maxCharges > 1 then
		button.Count:SetText(charges)
	else
		button.Count:SetText("")
	end
	if charges and charges > 0 and charges < maxCharges then
		button.CD:SetCooldown(chargeStart, chargeDuration)
		button.CD:Show()
		button.Icon:SetDesaturated(false)
		button.Count:SetTextColor(0, 1, 0)
	elseif start and duration > 1.5 then
		button.CD:SetCooldown(start, duration)
		button.CD:Show()
		button.Icon:SetDesaturated(true)
		button.Count:SetTextColor(1, 1, 1)
	else
		button.CD:Hide()
		button.Icon:SetDesaturated(false)
		if charges == maxCharges then button.Count:SetTextColor(1, 0, 0) end
	end

	if texture then
		button.Icon:SetTexture(GetSpellTexture(spellID))
	end
end

local function GetUnitAura(unit, spell, filter)
	for index = 1, 32 do
		local name, _, count, _, duration, expire, caster, _, _, spellID, _, _, _, _, _, value = UnitAura(unit, index, filter)
		if not name then break end
		if name and spellID == spell then
			return name, count, duration, expire, caster, spellID, value
		end
	end
end

local function UpdateAura(button, unit, auraID, filter, spellID, cooldown)
	button.Icon:SetTexture(GetSpellTexture(spellID))
	local name, count, duration, expire, caster = GetUnitAura(unit, auraID, filter)
	if name and caster == "player" then
		if button.Count then
			if count == 0 then count = "" end
			button.Count:SetText(count)
		end
		button.CD:SetCooldown(expire-duration, duration)
		button.CD:Show()
		button.Icon:SetDesaturated(false)
	else
		if cooldown then
			UpdateCooldown(button, spellID)
		else
			if button.Count then button.Count:SetText("") end
			button.CD:Hide()
			button.Icon:SetDesaturated(true)
		end
	end
end

local function UpdateTotemAura(button, texture, spellID)
	button.Icon:SetTexture(texture)
	local found
	for slot = 1, 4 do
		local haveTotem, _, start, dur, icon = GetTotemInfo(slot)
		if haveTotem and icon == texture and (start+dur-GetTime() > 0) then
			button.CD:SetCooldown(start, dur)
			button.CD:Show()
			button.Icon:SetDesaturated(false)
			button.Count:SetText("")
			found = true
			break
		end
	end
	if not found then
		if spellID then
			UpdateCooldown(button, spellID)
		else
			button.CD:Hide()
			button.Icon:SetDesaturated(true)
		end
	end
end

local faderEvents = {
	["UNIT_EXITED_VEHICLE"] = true,
	["UNIT_ENTERED_VEHICLE"] = true,
	["PLAYER_REGEN_ENABLED"] = true,
	["PLAYER_REGEN_DISABLED"] = true,
	["PLAYER_ENTERING_WORLD"] = true,
}

local f = CreateFrame("Frame")
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("PLAYER_TALENT_UPDATE")
f:SetScript("OnEvent", function(self, event)
	if event == "PLAYER_LOGIN" or event == "PLAYER_TALENT_UPDATE" then
		if GetSpecialization() == 1 then
			StaggerGo()

			f:RegisterUnitEvent("UNIT_AURA", "player")
			f:RegisterEvent("UNIT_MAXHEALTH")
			f:RegisterEvent("SPELL_UPDATE_COOLDOWN")
			f:RegisterEvent("SPELL_UPDATE_CHARGES")
			for event in pairs(faderEvents) do
				f:RegisterEvent(event)
			end
		else
			if bar then bar:Hide() end

			f:UnregisterEvent("UNIT_AURA")
			f:UnregisterEvent("UNIT_MAXHEALTH")
			f:UnregisterEvent("SPELL_UPDATE_COOLDOWN")
			f:UnregisterEvent("SPELL_UPDATE_CHARGES")
			for event in pairs(faderEvents) do
				f:UnregisterEvent(event)
			end
		end
	elseif faderEvents[event] then
		if event == "PLAYER_REGEN_DISABLED" or InCombatLockdown() then
			UIFrameFadeIn(bar, .3, bar:GetAlpha(), 1)
		else
			UIFrameFadeOut(bar, 2, bar:GetAlpha(), cfg.OOCAlpha)
		end
	else
		local cur = UnitStagger("player") or 0
		local max = UnitHealthMax("player")
		local perc = cur / max
		bar:SetValue(perc * 100)
		bar.Count:SetText(cfg.InfoColor..Numb(cur).." "..cfg.MyColor..Numb(perc * 100).."%")

		UpdateCooldown(bu[1], 121253, true)
		UpdateCooldown(bu[2], 322101, true)
		bu[2].Count:SetText(GetSpellCount(322101))
		UpdateAura(bu[3], "player", 215479, "HELPFUL", 215479)
		UpdateAura(bu[4], "player", 325092, "HELPFUL", 325092)
		UpdateAura(bu[5], "player", 322507, "HELPFUL", 322507, true)
	end

	if not InCombatLockdown() then
		for i = 1, 5 do
			bu[i].Count:SetTextColor(1, 1, 1)
			bu[i].Count:SetText("")
			bu[i].CD:Hide()
			bu[i]:SetScript("OnUpdate", nil)
			bu[i].Icon:SetDesaturated(true)
		end
	end
end)

